import { system,ItemStack,world,EquipmentSlot,EntityComponentTypes, TicksPerSecond } from "@minecraft/server";
import { RawtextBuilder, showGuidePage, showPage } from "./GuideBook";
import { ModalFormData, ActionFormData, MessageFormData } from "@minecraft/server-ui";

export const VERSION = "1.1.0";
export const VERSION_HISTORY = ["1.1.0"];

function removeSingleItem(item, player) {
  const equippable = player.getComponent("equippable");
  const mainHandItem = equippable.getEquipment(EquipmentSlot.Mainhand);

  if (mainHandItem && mainHandItem.typeId === item.typeId) {
    if ((item.amount == 1)) {
      equippable.setEquipment(
        EquipmentSlot.Mainhand,
        new ItemStack("minecraft:air")
      );
    } else {
      const newItemStack = new ItemStack(item.typeId, mainHandItem.amount - 1);
      equippable.setEquipment(EquipmentSlot.Mainhand, newItemStack);
    }
  }
}

system.runInterval(() => {
  for (const player of world.getAllPlayers()) {
    const inventory = player.getComponent(EntityComponentTypes.Inventory);

    if (inventory === undefined) return;
    const container = inventory.container;
    if (container === undefined) return;

    // Update inventory items with the typeId "sg_mail:guide"
    for (let i = 0; i < container.size; i++) {
      const itemStack = container.getItem(i);
      if (itemStack === undefined) continue;
      if (itemStack.typeId === "sg_mail:guide") {
        const lore = itemStack.getLore() || []; // Get the existing lore or start with an empty array
        if (!lore.includes("§r§7Bedrock Essential")) {
          lore.push("§r§7Bedrock Essential");
          itemStack.setLore(lore);
          container.setItem(i, itemStack);
        }
      }
    }
  }
});

const viewInboxForm = (toPlayer) => {
  const inboxForm = new ActionFormData().title("Inbox");

  // Fetch all objectives and filter the ones relevant to the player as the recipient
  const objectives = world.scoreboard.getObjectives();
  const messages = objectives
    .filter(objective => {
      const parts = objective.id.split(":");
      return parts.length === 5 && parts[2] === toPlayer.name;
    })
    .map(objective => objective.id.split(":"));

  if (messages.length === 0) {
    // If no messages, display a "no messages" text and a "Go Back" button
    inboxForm.body("§7You have no messages.");
  } else {
    // If there are messages, display them as buttons
    messages.forEach(messageParts => {
      const [_, subject, __, fromPlayer, message] = messageParts;
      inboxForm.button(`${subject} - ${fromPlayer}`, "textures/sg/mail/icons/text.png");
    });
  }

  // Add "Go Back" button
  inboxForm.button("§oGo Back");

  inboxForm.show(toPlayer).then((res) => {
    if (res.canceled || res.selection === undefined) return;

    if (messages.length === 0 || res.selection === messages.length) {
      // If no messages or "Go Back" is selected, return to the main page
      showMainPage(toPlayer);
    } else {
      const selectedMessageParts = messages[res.selection];
      const [_, subject, __, fromPlayer, message] = selectedMessageParts;

      viewMessageForm(toPlayer, subject, fromPlayer, message);
    }
  });
};

const viewMessageForm = (toPlayer, subject, fromPlayer, message) => {
  const messageForm = new ActionFormData()
    .title(`Message from ${fromPlayer}`)
    .body(
      `§7Subject: §f${subject}\n§7From: §f${fromPlayer}§r\n\n§f${message}§r\n\n\n§7What would you like to do?\n`
    )
    .button("Reply", "textures/sg/mail/icons/reply.png")
    .button("Delete", "textures/sg/mail/icons/delete.png")
    .button("§oGo Back");

  messageForm.show(toPlayer).then((res) => {
    if (res.canceled) return;

    if (res.selection === 0) {
      // Reply
      sendMessageForm(toPlayer, fromPlayer, subject);
    } else if (res.selection === 1) {
      // Delete
      const objectiveName = `sg_mail:${subject}:${toPlayer.name}:${fromPlayer}:${message}`;
      const objective = world.scoreboard.getObjective(objectiveName);
      if (objective) {
        world.scoreboard.removeObjective(objective.id);
        toPlayer.sendMessage(`§aMessage deleted.`);
      } else {
        toPlayer.sendMessage(`§cMessage not found.`);
      }
      viewInboxForm(toPlayer); // Go back to inbox after deleting
    } else if (res.selection === 2) {
      // Go Back
      viewInboxForm(toPlayer);
    }
  });
};

const viewOutboxForm = (fromPlayer) => {
  const outboxForm = new ActionFormData().title("Outbox");

  // Fetch all objectives and filter the ones where the player is the sender
  const objectives = world.scoreboard.getObjectives();
  const sentMessages = objectives
    .filter((objective) => {
      const parts = objective.id.split(":");
      return parts.length === 5 && parts[3] === fromPlayer.name;
    })
    .map((objective) => objective.id.split(":"));

  if (sentMessages.length === 0) {
    // If no messages, display a "no messages" text and a "Go Back" button
    outboxForm.body("§7You have not sent any messages.");
    outboxForm.button("§oGo Back");
  } else {
    // If there are messages, display them as buttons
    sentMessages.forEach((messageParts) => {
      const [_, subject, toPlayer] = messageParts;
      outboxForm.button(
        `${subject} - ${toPlayer}`,
        "textures/sg/mail/icons/text.png"
      );
    });

    // Add a "Go Back" button at the bottom
    outboxForm.button("§oGo Back");
  }

  outboxForm.show(fromPlayer).then((res) => {
    if (res.canceled || res.selection === undefined) return;

    if (sentMessages.length === 0 || res.selection === sentMessages.length) {
      // If no messages or the "Go Back" button is selected
      showMainPage(fromPlayer);
    } else {
      const selectedMessageParts = sentMessages[res.selection];
      const [_, subject, toPlayer, __, message] = selectedMessageParts;

      viewOutboxMessageForm(fromPlayer, subject, toPlayer, message);
    }
  });
};

const viewOutboxMessageForm = (fromPlayer, subject, toPlayer, message) => {
  const messageForm = new ActionFormData()
    .title(`Sent Message: ${subject}`)
    .body(`§7To: §f${toPlayer}§r\n§7Subject: §f${subject}\n\n§f${message}§r\n\n\n§7What would you like to do?`)
    .button("Undo Send", "textures/sg/mail/icons/delete.png")
    .button("Go Back");

  messageForm.show(fromPlayer).then((res) => {
    if (res.canceled || res.selection === undefined) return;

    if (res.selection === 0) {
      // Undo Send: Remove the scoreboard objective
      const objectiveName = `sg_mail:${subject}:${toPlayer}:${fromPlayer.name}:${message}`;
      const objective = world.scoreboard.getObjective(objectiveName);
      if (objective) {
        world.scoreboard.removeObjective(objective);
        fromPlayer.sendMessage(`§aMessage to ${toPlayer} with subject "${subject}" has been deleted.`);
      } else {
        fromPlayer.sendMessage(`§cFailed to undo send. The message could not be found.`);
      }
      viewOutboxForm(fromPlayer); // Go back to the outbox after undoing
    } else {
      viewOutboxForm(fromPlayer); // Just go back to the outbox
    }
  });
};

const sendMessageForm = (fromPlayer, prefilledToPlayer = "", subject = "") => {
  // Check if subject needs prefix
  const formattedSubject =
    subject && !subject.startsWith("Re;") ? `Re; ${subject}` : subject;

  const form = new ModalFormData()
    .title("Send Mail")
    .textField("Subject", "Enter subject here", formattedSubject)
    .textField("To Player", "Enter recipient's name", prefilledToPlayer)
    .textField("Message", "Enter your message\n\n", "");

  form.show(fromPlayer).then((res) => {
    if (res.canceled) return;

    const [subject, toPlayer, message] = res.formValues;

    // Check if any fields are empty
    if (!subject || !toPlayer || !message) {
      fromPlayer.sendMessage("§cAll fields must be filled out.");
      return;
    }

    // Check if any field contains ":"
    if (
      subject.includes(":") ||
      toPlayer.includes(":") ||
      message.includes(":")
    ) {
      fromPlayer.sendMessage(
        "§cYou cannot use the ':' character in the subject, recipient's name, or message."
      );
      return;
    }

    const objectiveName = `sg_mail:${subject}:${toPlayer}:${fromPlayer.name}:${message}`;

    // Check if the objective already exists
    const existingObjective = world.scoreboard.getObjective(objectiveName);
    if (!existingObjective) {
      // Create the scoreboard objective to store the message
      world.scoreboard.addObjective(objectiveName);
      fromPlayer.playSound("mob.phantom.flap", {
        location: fromPlayer.location,
        pitch: 2,
        volume: 1,
      });
      fromPlayer.playSound("mob.phantom.flap", {
        location: fromPlayer.location,
        pitch: 2.8,
        volume: 1,
      });

      // Check if the recipient is online
      const recipient = world
        .getPlayers()
        .find((player) => player.name === toPlayer);
      if (recipient) {
        recipient.playSound("mob.phantom.flap", {
          location: recipient.location,
          pitch: 2,
          volume: 1,
        });
        recipient.playSound("mob.phantom.flap", {
          location: recipient.location,
          pitch: 2.8,
          volume: 1,
        });
        recipient.sendMessage(
          `§eYou have received a new message with subject: §6${subject}`
        );
        fromPlayer.sendMessage(
          `§aMessage sent to ${toPlayer} with subject "${subject}".`
        );
      } else {
        fromPlayer.sendMessage(
          `§aMessage sent to ${toPlayer} with subject "${subject}", they aren't online currently, but they will see it when they return.`
        );
      }

      // Check if the recipient is registered
      const recipientObjective = world.scoreboard.getObjective(
        `sg_mail_player:${toPlayer}`
      );
      if (!recipientObjective) {
        fromPlayer.sendMessage(
          `§eThe message has been sent to "${toPlayer}" but our system indicates that this player has never joined the world.`
        );
      }
    } else {
      fromPlayer.sendMessage(`§cA message with this subject already exists.`);
    }
  });
};











world.afterEvents.playerSpawn.subscribe((event) => {
  const { initialSpawn, player } = event;
  const playerObjectiveName = `sg_mail_player:${player.name}`;
  const existingObjective = world.scoreboard.getObjective(playerObjectiveName);

  if (initialSpawn) {
    // If it's the player's first time joining with the add-on
    if (!existingObjective) {
      system.runTimeout(() => {
        player.sendMessage(
          "§aWelcome to the Mailbox Add-On!\n§eYou can send and receive messages from friends, whether they are online or offline.\nUse the Mailbox Handbook to start sending mail!"
        );

        // Add the player to a scoreboard objective to track that they have joined
        world.scoreboard.addObjective(playerObjectiveName);

        // Create a welcome message for the player
        const welcomeSubject = "Welcome to Mailbox";
        const welcomeMessage =
          "Thank you for using the Mailbox Add-On!\nNow you can easily send and receive messages from other players,\neven when they are offline. Enjoy your time!";
        const welcomeObjectiveName = `sg_mail:${welcomeSubject}:${player.name}:SkyGames:${welcomeMessage}`;

        // Check if the welcome message objective already exists
        const welcomeObjective =
          world.scoreboard.getObjective(welcomeObjectiveName);
        if (!welcomeObjective) {
          world.scoreboard.addObjective(welcomeObjectiveName);
          player.sendMessage(
            "§aYou have received a welcome message in your inbox from SkyGames with the subject 'Welcome to Mailbox'."
          );
        }
      }, 10 * TicksPerSecond);
    }

    // Handle version check and guide delivery as before
    const version = player.getDynamicProperty("sg_mail_version");
    if (version === undefined) {
      player.setDynamicProperty("sg_mail_version", VERSION);
      if (player.hasTag("sg_mail_received_guide")) {
        player.removeTag("sg_mail_received_guide");
        player.sendMessage({
          translate: "sg_mail.announce_update",
          with: [VERSION],
        });
        return;
      }
      player.dimension.spawnItem(
        new ItemStack("sg_mail:guide"),
        player.location
      );
    } else if (version !== VERSION) {
      player.setDynamicProperty("sg_mail_version", VERSION);
      player.sendMessage({
        translate: "sg_mail.announce_update",
        with: [VERSION],
      });
    }
  } else {
    system.runTimeout(() => {
      // If the player has joined before and is spawning again (not initial spawn)
      const totalMessageNumber = getTotalMessagesForPlayer(player); // You need to implement this function to retrieve the number of messages

      // Determine the correct pluralization
      const messagePlural = totalMessageNumber === 1 ? "message" : "messages";
      player.sendMessage(
        `§aWelcome back! You have ${totalMessageNumber} ${messagePlural} in your Mailbox.`
      );
    },8*TicksPerSecond);
  }
});

function getTotalMessagesForPlayer(player) {
  const objectives = world.scoreboard.getObjectives();
  const playerName = player.name;

  // Filter objectives where the player's name is in the "To" slot (third part of the ID)
  const receivedMessages = objectives.filter((objective) => {
    const parts = objective.id.split(":");
    return parts.length === 5 && parts[2] === playerName; // Assuming "To" is the third part
  });

  // The total number of received messages
  return receivedMessages.length;
}







const BUTTONS = [
  {
    // SEND MESSAGE
    translation_key: "sg_mail.guide.send_message.title",
    icon_path: "textures/sg/mail/icons/add.png",
    action: (player) => {
      sendMessageForm(player); // Opens the form to send a message
    },
  },
  {
    // INBOX
    translation_key: "sg_mail.guide.inbox.title",
    icon_path: "textures/sg/mail/icons/inbox.png",
    action: (player) => {
      viewInboxForm(player); // Opens the inbox form with the message count
    },
  },
  {
    // OUTBOX
    translation_key: "sg_mail.guide.outbox.title",
    icon_path: "textures/sg/mail/icons/outbox.png",
    action: (player) => {
      viewOutboxForm(player); // Opens the inbox form with the message count
    },
  },
  {
    // CHANGELOG
    translation_key: "sg_mail.guide.changelog.title",
    icon_path: "textures/ui/book_addtextpage_default.png",
    action: (player) => {
      const body = new RawtextBuilder();
      body.pushLine({
        translate: "sg_mail.guide.changelog.body",
        with: [VERSION],
      });
      body.pushLine();
      for (const version of VERSION_HISTORY) {
        body.pushLine({
          translate: "sg_mail.guide.changelog.subtitle",
          with: [version],
        });
        body.pushLine();
        body.pushLine({
          translate: `sg_mail.guide.changelog.${version}`,
          with: ["\n"],
        });
        body.pushLine();
        body.pushLine();
      }
      showPage(player, "sg_mail.guide.changelog.title", body.build(), [
        {
          translation_key: "sg_mail.guide.common.back_button",
          action: (player) => showMainPage(player),
        },
      ]);
    },
  },
  {
    translation_key: "sg_mail.guide.common.exit_button",
  },
];

const showMainPage = (player) => {
  showGuidePage(player, "main", 4, BUTTONS);
};

world.afterEvents.itemUse.subscribe((event) => {
  system.run(() => {
    if (event.itemStack.typeId === "sg_mail:guide") {
      showMainPage(event.source);
    }
  });
});